# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Django settings for defivelo project.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

import os

from django.utils.translation import gettext_lazy as _

import dj_database_url
import dj_email_url

from .. import get_project_root_path
from . import get_env_variable


def gettext(s):
    return s


PROJECT_ROOT = get_project_root_path()

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = get_env_variable("SECRET_KEY", "")

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = bool(get_env_variable("DEBUG", False))

ALLOWED_HOSTS = tuple(get_env_variable("ALLOWED_HOSTS", "").splitlines())

SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

USE_X_FORWARDED_HOST = get_env_variable("USE_X_FORWARDED_HOST", False)

# Application definition

UPSTREAM_APPS = (
    "bootstrap3",
    "bootstrap3_datetime",
    "django.contrib.sites",
    "django.contrib.postgres",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "dal",
    "dal_select2",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "crispy_forms",
    "crispy_bootstrap3",
    "compressor",
    "stronghold",
    "localflavor",
    "phonenumber_field",
    "parler",
    "django_countries",
    "django_filters",
    "import_export",
    "django_agpl",
    "tinymce",
    "taggit",
    "rolepermissions",
    "simple_history",
    "memoize",
)

CRISPY_ALLOWED_TEMPLATE_PACKS = "bootstrap3"
CRISPY_TEMPLATE_PACK = "bootstrap3"

# Project apps tested by jenkins (everything in apps/)
APPS_DIR = os.path.join(PROJECT_ROOT, "apps")
EXCLUDED_DIRS = "__pycache__"
PROJECT_APPS = tuple(
    [
        "apps." + aname
        for aname in os.listdir(APPS_DIR)
        if os.path.isdir(os.path.join(APPS_DIR, aname)) and aname not in EXCLUDED_DIRS
    ]
)

INSTALLED_APPS = PROJECT_APPS + UPSTREAM_APPS + ("defivelo",)

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "allauth.account.middleware.AccountMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "stronghold.middleware.LoginRequiredMiddleware",
    "simple_history.middleware.HistoryRequestMiddleware",
]

ROOT_URLCONF = "defivelo.urls"

WSGI_APPLICATION = "defivelo.wsgi.application"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {"console": {"class": "logging.StreamHandler"}},
    "root": {
        "handlers": ["console"],
        "level": "INFO",
    },
    "loggers": {
        "apps": {"handlers": ["console"], "level": "DEBUG", "propagate": True},
        "django": {"handlers": ["console"], "level": "WARNING", "propagate": True},
        "werkzeug": {"handlers": ["console"], "level": "DEBUG", "propagate": True},
    },
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(PROJECT_ROOT, "defivelo/templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                "defivelo.context_processors.now",
            ],
            "debug": DEBUG,
        },
    },
]

SASSC_BIN_PATH = get_env_variable("SASSC_BIN_PATH", "/usr/bin/sassc")

COMPRESS_PRECOMPILERS = (
    (
        "text/x-scss",
        SASSC_BIN_PATH + " {infile} {outfile}",
    ),
)

COMPRESS_FILTERS = {
    "css": [
        "compressor.filters.css_default.CssAbsoluteFilter",
        "compressor.filters.cssmin.CSSCompressorFilter",
    ],
    "js": ["compressor.filters.jsmin.JSMinFilter"],
}
# Only allow online compression because we are not able to pre-compress some dynamic assets
# (see declarations of "bootstrap3_extra_script" block in templates)
COMPRESS_OFFLINE = False
COMPRESS_ENABLED = True

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases
DATABASES = {"default": dj_database_url.parse(get_env_variable("DATABASE_URL"))}
# Allow using a password from a k8s secret
DATABASE_PASSWORD = get_env_variable("DATABASE_PASSWORD", False)
if DATABASE_PASSWORD:
    DATABASES["default"]["PASSWORD"] = DATABASE_PASSWORD


SITE_ID = 1

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/
# Internationalization
LANGUAGE_CODE = "fr"
TIME_ZONE = "Europe/Zurich"
USE_I18N = True
USE_TZ = True

DATE_FORMAT = "j F Y"
DATE_FORMAT_SHORT = "j.m"
DATE_FORMAT_COMPACT = "j.m.y"
TIME_FORMAT = r"G\hi"
TIME_FORMAT_SHORT = "G:i"

LANGUAGES = (
    ("fr", gettext("French")),
    ("de", gettext("German")),
)

COUNTRIES_FIRST = [
    "CH",
]

PHONENUMBER_DEFAULT_REGION = "CH"

PARLER_LANGUAGES = {SITE_ID: ([{"code": lang[0]} for lang in LANGUAGES]), "default": {}}

# This allows you to put project-wide translations in the "locale" directory of
# your project
LOCALE_PATHS = (os.path.join(PROJECT_ROOT, "locale"),)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/
# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = get_env_variable("STATIC_URL", "/static/")

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# This is usually not used in a dev env, hence the default value
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = get_env_variable(
    "STATIC_ROOT", os.path.join(PROJECT_ROOT, "static_files")
)

STATICFILES_DIRS = (
    os.path.join(PROJECT_ROOT, "defivelo/static"),
    os.path.join(PROJECT_ROOT, "ext/bootstrap-datetimepicker/build/"),
    os.path.join(PROJECT_ROOT, "ext/moment/min/"),
)

STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder",
)

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = get_env_variable("MEDIA_URL", "/media/")

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = get_env_variable("MEDIA_ROOT", "/tmp/static/media")

# Adapt Stronghold for allauth
STRONGHOLD_PUBLIC_URLS = [
    r"^/admin/.*$",  # Administration
    r"^/accounts/.*$",  # Login
    r"^/agpl-.*$",  # License
]

AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
)
ACCOUNT_AUTHENTICATION_METHOD = "email"
ACCOUNT_EMAIL_VERIFICATION = "mandatory"
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = True

TINYMCE_DEFAULT_CONFIG = {
    "theme": "silver",
    "height": 500,
    "menubar": False,
    "plugins": ",".join(
        [
            "advlist",
            "autolink",
            "lists",
            "link",
            "image",
            "charmap",
            "print",
            "preview",
            "anchor",
            "searchreplace",
            "visualblocks",
            "code",
            "fullscreen",
            "insertdatetime",
            "media",
            "table",
            "paste",
            "code",
            "help",
            "wordcount",
        ]
    ),
    "toolbar": " | ".join(
        [
            " ".join(block)
            for block in [
                [
                    "formatselect",
                ],
                [
                    "bold",
                    "italic",
                    "underline",
                    "strikethrough",
                ],
                [
                    "undo",
                    "redo",
                ],
                [
                    "link",
                    "unlink",
                ],
                [
                    "alignleft",
                    "aligncenter",
                    "alignright",
                    "alignjustify",
                ],
                ["bullist", "numlist"],
                ["removeformat"],
            ]
        ]
    ),
    "branding": False,
}
TINYMCE_JS_URL = os.path.join(STATIC_URL, "tinymce/tinymce.min.js")
TINYMCE_JS_ROOT = os.path.join(STATIC_ROOT, "tinymce")
TINYMCE_INCLUDE_JQUERY = False
TINYMCE_COMPRESSOR = False


def defivelo_user_display(u):
    if u.first_name and u.last_name:
        return "{first} {last}".format(first=u.first_name, last=u.last_name)
    else:
        return u.email


ACCOUNT_USER_DISPLAY = defivelo_user_display
ACCOUNT_ADAPTER = "defivelo.accounts.NoSignupAccountAdapter"

SENTRY_DSN = get_env_variable("SENTRY_DSN", "")

if SENTRY_DSN:
    import sentry_sdk
    from sentry_sdk.integrations.django import DjangoIntegration

    sentry_sdk.init(
        dsn=SENTRY_DSN,
        integrations=[DjangoIntegration()],
        environment=get_env_variable("SENTRY_ENVIRONMENT", ""),
    )


ROLEPERMISSIONS_MODULE = "defivelo.roles"

LOGIN_REDIRECT_URL = "/"

DATA_UPLOAD_MAX_NUMBER_FIELDS = 2048  # Up from '1000'

VCS_VERSION = get_env_variable("VCS_VERSION", "0")
VCS_COMMIT = get_env_variable("VCS_COMMIT", "0")

if VCS_VERSION == "0":
    import subprocess

    try:
        VCS_VERSION = subprocess.check_output(
            ["git", "describe", "HEAD"], encoding="utf8"
        )
        VCS_COMMIT = subprocess.check_output(
            ["git", "rev-parse", "HEAD"], encoding="utf8"
        )
    except (IOError, subprocess.CalledProcessError, UnicodeDecodeError):
        VCS_VERSION = "undefined"
        VCS_COMMIT = "HEAD"

try:
    from django_agpl import app_settings as agpl_settings

    # Base directory from which download tree will be generated
    AGPL_ROOT = PROJECT_ROOT

    # Prefix of generated filename
    AGPL_FILENAME_PREFIX = "defivelo-intranet"

    # Directories to exclude from download tree (optional)
    AGPL_EXCLUDE_DIRS = agpl_settings.EXCLUDE_DIRS + [
        r"\.tox$",
        r"\.kdev4$",
        r"^__pycache__$",
        r"\.vagrant$",
        r"^virtualization",
        r"^ext",
        r"^static_files$",
        r"^venv$",
        r"^envdir$",
    ]

    # Files to exclude from download tree (optional)
    AGPL_EXCLUDE_FILES = agpl_settings.EXCLUDE_FILES + [
        r".mo$",
        r"~$",
        r"pgdump$",
        r"pg$",
        r"kdev4$",
    ]

    # Prefix to create inside download tree (optional)
    AGPL_TREE_PREFIX = "defivelo-intranet"
except ImportError:
    pass  # Sorry


BOOTSTRAP3 = {
    "required_css_class": "required-field",
    "field_renderers": {
        "default": "defivelo.bootstrap3.DvFieldRenderer",
        "inline": "defivelo.bootstrap3.DvInlineFieldRenderer",
    },
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

#############
# E-Mailing #
#############
EMAIL_URL = get_env_variable("EMAIL_URL", "console://")
email_config = dj_email_url.parse(EMAIL_URL)

# Email sender settings
EMAIL_SUBJECT_PREFIX = _("DÉFI VÉLO: ")
SERVER_EMAIL = email_config.get(
    "SERVER_EMAIL", get_env_variable("SERVER_EMAIL", "noreply@defi-velo.ch")
)
DEFAULT_FROM_EMAIL = email_config.get(
    "DEFAULT_FROM_EMAIL", get_env_variable("DEFAULT_FROM_EMAIL", "noreply@defi-velo.ch")
)
PROFILE_CHANGED_NOTIFY_EMAIL = get_env_variable(
    "PROFILE_CHANGED_NOTIFY_EMAIL", "office@defi-velo.ch"
)
# Email backend settings
EMAIL_HOST = email_config["EMAIL_HOST"] or "localhost"
EMAIL_PORT = email_config["EMAIL_PORT"] or 25
EMAIL_FILE_PATH = email_config["EMAIL_FILE_PATH"]
EMAIL_HOST_USER = email_config["EMAIL_HOST_USER"]
EMAIL_HOST_PASSWORD = email_config["EMAIL_HOST_PASSWORD"]
EMAIL_BACKEND = email_config["EMAIL_BACKEND"]
EMAIL_USE_TLS = email_config["EMAIL_USE_TLS"]
EMAIL_USE_SSL = email_config["EMAIL_USE_SSL"]



# Generated by Django 1.11.4 on 2017-09-04 08:58
from __future__ import unicode_literals

from django.db import migrations, models

from apps.common import DV_SEASON_AUTUMN, DV_SEASON_STATE_ARCHIVED, DV_SEASON_STATE_OPEN


def make_all_seasons_visible(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Season = apps.get_model("challenge", "Season")
    for season in Season.objects.all():
        if season.year == 2017 and season.season == DV_SEASON_AUTUMN:
            season.state = DV_SEASON_STATE_OPEN
        else:
            season.state = DV_SEASON_STATE_ARCHIVED
        season.save()


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0043_VS-OW_to_WS"),
    ]

    operations = [
        migrations.AddField(
            model_name="season",
            name="state",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (1, "Planification (invisible)"),
                    (2, "Annonc\xe9e (rentr\xe9e des disponibilit\xe9s)"),
                    (3, "En cours (corrections que par charg\xe9\xb7e\xb7s de projet)"),
                    (4, "Termin\xe9e"),
                    (5, "Archiv\xe9e"),
                ],
                default=1,
                verbose_name="\xc9tat",
            ),
        ),
        migrations.RunPython(make_all_seasons_visible, migrations.RunPython.noop),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015-2020 Didier Raboud <didier.raboud>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from .availability import HelperSessionAvailability
from .invoices import Invoice, InvoiceLine
from .qualification import Qualification, QualificationActivity
from .season import Season
from .session import Session
from .settings import AnnualStateSetting

__all__ = [
    "AnnualStateSetting",
    "HelperSessionAvailability",
    "Invoice",
    "InvoiceLine",
    "Qualification",
    "QualificationActivity",
    "Season",
    "Session",
]



# Generated by Django 2.2.16 on 2020-10-28 13:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0071_remove_season"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalseason",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Mois",
            },
        ),
        migrations.AlterModelOptions(
            name="season",
            options={
                "ordering": ["year", "month_start", "cantons"],
                "verbose_name": "Mois",
                "verbose_name_plural": "Mois",
            },
        ),
        migrations.AlterField(
            model_name="helperseasonworkwish",
            name="season",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="work_wishes",
                to="challenge.Season",
                verbose_name="Mois",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelperseasonworkwish",
            name="season",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.Season",
                verbose_name="Mois",
            ),
        ),
        migrations.AlterField(
            model_name="invoice",
            name="season",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="invoices",
                to="challenge.Season",
                verbose_name="Mois",
            ),
        ),
    ]



from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag
def is_supported_email_backend():
    return settings.EMAIL_BACKEND in (
        "django.core.mail.backends.locmem.EmailBackend",
        "django.core.mail.backends.filebased.EmailBackend",
    )



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0006_auto_20150901_1210"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="SessionActivity",
            new_name="QualificationActivity",
        ),
        migrations.RenameModel(
            old_name="SessionActivityTranslation",
            new_name="QualificationActivityTranslation",
        ),
        migrations.AddField(
            model_name="qualification",
            name="leader",
            field=models.ForeignKey(
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur 2",
                null=True,
                blank=True,
                related_name="sessions_mon2",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterModelTable(
            name="qualificationactivitytranslation",
            table="challenge_qualificationactivity_translation",
        ),
    ]



from django import forms
from django.contrib.postgres.fields import ArrayField
from django.forms import widgets


class TimeNumberInput(widgets.NumberInput):
    input_type = "number"
    template_name = "forms/widgets/timenumber.html"


class NumberInput(widgets.NumberInput):
    input_type = "number"
    template_name = "forms/widgets/number.html"


# overide to make compatible with bootstrap
class CheckboxInput(widgets.Input):
    input_type = "checkbox"
    template_name = "django/forms/widgets/checkbox.html"

    def __init__(self, attrs=None, check_test=None):
        super().__init__(attrs)
        # check_test is a callable that takes a value and returns True
        # if the checkbox should be checked for that value.
        self.check_test = widgets.boolean_check if check_test is None else check_test

    def format_value(self, value):
        """Only return the 'value' attribute if value isn't empty."""
        if value is True or value is False or value is None or value == "":
            return
        return str(value)

    def get_context(self, name, value, attrs):
        if self.check_test(value):
            attrs = {**(attrs or {}), "checked": True}
        if "class" in attrs:
            attrs["class"] = attrs["class"].replace("form-control", "").strip()
        return super().get_context(name, value, attrs)

    def value_from_datadict(self, data, files, name):
        if name not in data:
            # A missing value means False because HTML form submission does not
            # send results for unselected checkboxes.
            return False
        value = data.get(name)
        # Translate true and false strings to boolean values.
        values = {"true": True, "false": False}
        if isinstance(value, str):
            value = values.get(value.lower(), value)
        return bool(value)

    def value_omitted_from_data(self, data, files, name):
        # HTML checkboxes don't appear in POST data if not checked, so it's
        # never known if the value is actually omitted.
        return False


class CheckboxMultipleChoiceField(forms.MultipleChoiceField):
    widget = forms.CheckboxSelectMultiple


class ChoiceArrayField(ArrayField):
    """
    A field that allows us to store an array of choices.
    Uses Django's Postgres ArrayField
    and a MultipleChoiceField for its formfield.
    """

    def formfield(self, **kwargs):
        defaults = {
            "form_class": CheckboxMultipleChoiceField,
            "choices": self.base_field.choices,
        }

        defaults.update(kwargs)
        # Skip the parent's formfield implementation completely as it does not matter.
        return super(ArrayField, self).formfield(**defaults)



# Generated by Django 2.2.9 on 2020-04-07 07:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0003_auto_20200306_1015"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timesheet",
            name="time_actor",
            field=models.FloatField(default=0, verbose_name="Intervention(s)"),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from factory import Faker
from factory.django import DjangoModelFactory
from factory.fuzzy import FuzzyChoice

from apps.common import DV_STATES

from ..models import Organization


class OrganizationFactory(DjangoModelFactory):
    class Meta:
        model = Organization

    name = Faker("name")
    address_street = Faker("street_name")
    address_no = Faker("random_int", min=0, max=999)
    address_zip = Faker("random_int", min=1000, max=9999)
    address_city = Faker("city")
    address_canton = FuzzyChoice(DV_STATES)



# Generated by Django 2.2.12 on 2020-05-18 13:09

from django.db import migrations

wanted_urls = {
    "https://dropbox.com/?": "Factures Logistiques (Dropbox)",
    "https://sbb.ch/?": "Frais de trajet (CFF Business)",
}


def add_initial_validation_urls(apps, schema_editor):
    # We can't import the MonthlyCantonalValidationUrl model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    MCVU = apps.get_model("salary", "MonthlyCantonalValidationUrl")
    for url, name in wanted_urls.items():
        m = MCVU()
        m.set_current_language("fr")
        m.url = url
        m.name = name
        m.save()


def empty_validation_urls(apps, schema_editor):
    # We can't import the MonthlyCantonalValidationUrl model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    MCVU = apps.get_model("salary", "MonthlyCantonalValidationUrl")
    MCVU.objects.filter(url__in=wanted_urls.keys()).delete()


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0010_monthlycantonalvalidation_validated_urls"),
    ]

    operations = [
        migrations.RunPython(add_initial_validation_urls, empty_validation_urls)
    ]



from django.utils.translation import gettext_lazy as _

from import_export import fields, resources

from apps.challenge.models import Invoice


class InvoiceResource(resources.ModelResource):
    canton = fields.Field(
        column_name=_("Canton de la facture"), attribute="organization__address_canton"
    )
    month = fields.Field(
        column_name=_("Mois de la facture"), attribute="month_of_the_invoice"
    )
    invoice_nb = fields.Field(column_name=_("N° de la facture"), attribute="ref")
    org_name = fields.Field(
        column_name=_("Nom de l'établissement"), attribute="organization__name"
    )
    total_participants = fields.Field(
        column_name=_("Total participant·e·s"), attribute="sum_nb_participants"
    )
    cost_per_participant = fields.Field(
        column_name=_("Coût par participant·e·s"),
        attribute="settings__cost_per_participant",
    )
    total_cost_participants = fields.Field(
        column_name=_("Total coût participant·e·s"), attribute="sum_cost_participants"
    )
    total_nb_bikes = fields.Field(
        column_name=_("Total vélos"), attribute="adjusted_sum_of_bikes"
    )
    cost_per_bike = fields.Field(
        column_name=_("Coût par vélo"), attribute="settings__cost_per_bike"
    )
    total_cost_bikes = fields.Field(
        column_name=_("Total coût vélos"), attribute="sum_cost_bikes_reduced"
    )
    invoice_total = fields.Field(
        column_name=_("Montant total de la facture"), attribute="sum_cost"
    )

    class Meta:
        model = Invoice
        fields = "__all__"



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include
from django.urls import re_path
from django.views.decorators.cache import never_cache

from .views import (
    ResendUserCredentials,
    SendUserCredentials,
    UserAssignRole,
    UserCreate,
    UserDetail,
    UserList,
    UserListExport,
    UserUpdate,
)
from .views.actions import MarkInactive
from .views.autocomplete import (
    Actors,
    AllCoordinators,
    AllPersons,
    Helpers,
    Leaders,
    PersonsRelevantForSessions,
)
from .views.deletes import UserDelete

urlpatterns = [
    re_path(r"^$", never_cache(UserList.as_view()), name="user-list"),
    re_path(
        r"^(?P<void>)?(?P<format>[a-z]+)export/$",
        never_cache(UserListExport.as_view()),
        name="user-list-export",
    ),
    re_path(r"^new/$", UserCreate.as_view(), name="user-create"),
    re_path(r"^me/$", never_cache(UserDetail.as_view()), name="profile-detail"),
    re_path(
        r"^(?P<pk>[0-9]+)/$", never_cache(UserDetail.as_view()), name="user-detail"
    ),
    re_path(
        r"^(?P<pk>[0-9]+)/sendcreds/$",
        never_cache(SendUserCredentials.as_view()),
        name="user-sendcredentials",
    ),
    re_path(
        r"^(?P<pk>[0-9]+)/resendcreds/$",
        never_cache(ResendUserCredentials.as_view()),
        name="user-resendcredentials",
    ),
    re_path(
        r"^(?P<pk>[0-9]+)/delete/$",
        never_cache(UserDelete.as_view()),
        name="user-delete",
    ),
    re_path(
        r"^(?P<pk>[0-9]+)/assign_role/$",
        never_cache(UserAssignRole.as_view()),
        name="user-assign-role",
    ),
    re_path(
        r"^actions/",
        include(
            [
                re_path(
                    r"^mark-inactive$",
                    never_cache(MarkInactive.as_view()),
                    name="users-actions-markinactive",
                ),
            ]
        ),
    ),
    re_path(r"^(?P<pk>[0-9]+)/update/$", UserUpdate.as_view(), name="user-update"),
    re_path(r"^ac/all/$", AllPersons.as_view(), name="user-AllPersons-ac"),
    re_path(r"^ac/coordinator/$", AllCoordinators.as_view(), name="user-coordinators"),
    re_path(
        r"^ac/prfs/$",
        PersonsRelevantForSessions.as_view(),
        name="user-PersonsRelevantForSessions-ac",
    ),
    re_path(r"^ac/helpers/$", Helpers.as_view(), name="user-Helpers-ac"),
    re_path(r"^ac/leaders/$", Leaders.as_view(), name="user-Leaders-ac"),
    re_path(r"^ac/actors/$", Actors.as_view(), name="user-Actors-ac"),
]



# Generated by Django 1.11.4 on 2017-09-04 12:01
from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import revoke_permission
from rolepermissions.roles import assign_role, clear_roles


def reset_power_roles(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            clear_roles(user)
            assign_role(user, "power_user")
        if has_role(user, "state_manager"):
            clear_roles(user)
            assign_role(user, "state_manager")


def revoke_season_see_all_from_power_roles(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user") or has_role(user, "state_manager"):
            revoke_permission(user, "challenge_season_see_state_planning")
            user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0043_allow_poweruser_to_set_roles"),
    ]

    operations = [
        migrations.RunPython(reset_power_roles, revoke_season_see_all_from_power_roles),
    ]
